@@ -1,4 +1,4 @@
-#!/bin/sh
+#!/bin/bash
 set -eux
 
 if [ "${LIMA_CIDATA_CONTAINERD_SYSTEM}" != 1 ] && [ "${LIMA_CIDATA_CONTAINERD_USER}" != 1 ]; then
@@ -8,7 +8,22 @@ fi
 # This script does not work unless systemd is available
 command -v systemctl >/dev/null 2>&1 || exit 0
 
-if [ ! -x /usr/local/bin/nerdctl ]; then
+# Extract bin/nerdctl and compare whether it is newer than the current /usr/local/bin/nerdctl (if already exists).
+# Takes 4-5 seconds. (FIXME: optimize)
+tmp_extract_nerdctl="$(mktemp -d)"
+tar Cxzf "${tmp_extract_nerdctl}" "${LIMA_CIDATA_MNT}"/nerdctl-full.tgz bin/nerdctl
+
+if [ ! -f /usr/local/bin/nerdctl ] || [[ "${tmp_extract_nerdctl}"/bin/nerdctl -nt /usr/local/bin/nerdctl ]]; then
+	if [ -f /usr/local/bin/nerdctl ]; then
+		(
+			set +e
+			echo "Upgrading existing nerdctl"
+			echo "- Old: $(/usr/local/bin/nerdctl --version)"
+			echo "- New: $("${tmp_extract_nerdctl}"/bin/nerdctl --version)"
+			systemctl disable --now containerd buildkit stargz-snapshotter
+			sudo -iu "${LIMA_CIDATA_USER}" "XDG_RUNTIME_DIR=/run/user/${LIMA_CIDATA_UID}" "PATH=${PATH}" containerd-rootless-setuptool.sh uninstall
+		)
+	fi
 	tar Cxzf /usr/local "${LIMA_CIDATA_MNT}"/nerdctl-full.tgz
 
 	mkdir -p /etc/bash_completion.d
@@ -16,14 +31,29 @@ if [ ! -x /usr/local/bin/nerdctl ]; then
 	# TODO: enable zsh completion too
 fi
 
+rm -rf "${tmp_extract_nerdctl}"
+
+: "${CONTAINERD_NAMESPACE:=default}"
+# Overridable in .bashrc
+: "${CONTAINERD_SNAPSHOTTER:=overlayfs}"
+
 if [ "${LIMA_CIDATA_CONTAINERD_SYSTEM}" = 1 ]; then
-	mkdir -p /etc/containerd
+	mkdir -p /etc/containerd /etc/buildkit
 	cat >"/etc/containerd/config.toml" <<EOF
   version = 2
   [proxy_plugins]
     [proxy_plugins."stargz"]
       type = "snapshot"
       address = "/run/containerd-stargz-grpc/containerd-stargz-grpc.sock"
+EOF
+	cat >"/etc/buildkit/buildkitd.toml" <<EOF
+[worker.oci]
+  enabled = false
+
+[worker.containerd]
+  enabled = true
+  namespace = "${CONTAINERD_NAMESPACE}"
+  snapshotter = "${CONTAINERD_SNAPSHOTTER}"
 EOF
 	systemctl enable --now containerd buildkit stargz-snapshotter
 fi
@@ -55,14 +85,19 @@ EOF
 		fi
 		sudo -iu "${LIMA_CIDATA_USER}" "XDG_RUNTIME_DIR=/run/user/${LIMA_CIDATA_UID}" systemctl --user enable --now dbus
 		sudo -iu "${LIMA_CIDATA_USER}" "XDG_RUNTIME_DIR=/run/user/${LIMA_CIDATA_UID}" "PATH=${PATH}" containerd-rootless-setuptool.sh install
-		sudo -iu "${LIMA_CIDATA_USER}" "XDG_RUNTIME_DIR=/run/user/${LIMA_CIDATA_UID}" "PATH=${PATH}" containerd-rootless-setuptool.sh install-buildkit
-		sudo -iu "${LIMA_CIDATA_USER}" "XDG_RUNTIME_DIR=/run/user/${LIMA_CIDATA_UID}" "PATH=${PATH}" containerd-rootless-setuptool.sh install-fuse-overlayfs
-		if grep -q "release 8" /etc/system-release; then
-			echo >&2 "WARNING: the guest seems EL8-compatible OS. Skipping installing rootless stargz"
+		sudo -iu "${LIMA_CIDATA_USER}" "XDG_RUNTIME_DIR=/run/user/${LIMA_CIDATA_UID}" "PATH=${PATH}" \
+			"CONTAINERD_NAMESPACE=${CONTAINERD_NAMESPACE}" "CONTAINERD_SNAPSHOTTER=${CONTAINERD_SNAPSHOTTER}" \
+			containerd-rootless-setuptool.sh install-buildkit-containerd
+
+		# $CONTAINERD_SNAPSHOTTER is configured in 20-rootless-base.sh, when the guest kernel is < 5.13, or the instance was created with Lima < 0.9.0.
+		if [ "$(sudo -iu "${LIMA_CIDATA_USER}" sh -ec 'echo $CONTAINERD_SNAPSHOTTER')" = "fuse-overlayfs" ]; then
+			sudo -iu "${LIMA_CIDATA_USER}" "XDG_RUNTIME_DIR=/run/user/${LIMA_CIDATA_UID}" "PATH=${PATH}" containerd-rootless-setuptool.sh install-fuse-overlayfs
+		fi
+
+		if compare_version.sh "$(uname -r)" -ge "5.13"; then
+			sudo -iu "${LIMA_CIDATA_USER}" "XDG_RUNTIME_DIR=/run/user/${LIMA_CIDATA_UID}" "PATH=${PATH}" containerd-rootless-setuptool.sh install-stargz
 		else
-			if ! sudo -iu "${LIMA_CIDATA_USER}" "XDG_RUNTIME_DIR=/run/user/${LIMA_CIDATA_UID}" "PATH=${PATH}" containerd-rootless-setuptool.sh install-stargz; then
-				echo >&2 "WARNING: rootless stargz does not seem supported on this host (kernel older than 5.11?)"
-			fi
+			echo >&2 "WARNING: the guest kernel seems older than 5.13. Skipping installing rootless stargz."
 		fi
 		if [ -n "$selinux" ]; then
 			echo "Restoring SELinux"
