@@ -4,7 +4,6 @@ import (
 	"errors"
 	"fmt"
 	"io"
-	"io/fs"
 	"os"
 	"path/filepath"
 	"regexp"
@@ -12,6 +11,7 @@ import (
 	"strings"
 	"time"
 
+	"github.com/docker/go-units"
 	"github.com/lima-vm/lima/pkg/iso9660util"
 	"github.com/lima-vm/lima/pkg/limayaml"
 	"github.com/lima-vm/lima/pkg/localpathutil"
@@ -19,6 +19,7 @@ import (
 	qemu "github.com/lima-vm/lima/pkg/qemu/const"
 	"github.com/lima-vm/lima/pkg/sshutil"
 	"github.com/lima-vm/lima/pkg/store/filenames"
+	"github.com/lima-vm/lima/pkg/usrlocalsharelima"
 	"github.com/sirupsen/logrus"
 )
 
@@ -120,12 +121,44 @@ func GenerateISO9660(instDir, name string, y *limayaml.LimaYAML, udpDNSLocalPort
 		args.SSHPubKeys = append(args.SSHPubKeys, f.Content)
 	}
 
-	for _, f := range y.Mounts {
+	var fstype string
+	switch *y.MountType {
+	case limayaml.REVSSHFS:
+		fstype = "sshfs"
+	case limayaml.NINEP:
+		fstype = "9p"
+	}
+	for i, f := range y.Mounts {
+		tag := fmt.Sprintf("mount%d", i)
 		expanded, err := localpathutil.Expand(f.Location)
 		if err != nil {
 			return err
 		}
-		args.Mounts = append(args.Mounts, expanded)
+		options := "defaults"
+		if fstype == "9p" {
+			options = "ro"
+			if *f.Writable {
+				options = "rw"
+			}
+			options += ",trans=virtio"
+			options += fmt.Sprintf(",version=%s", *f.NineP.ProtocolVersion)
+			msize, err := units.RAMInBytes(*f.NineP.Msize)
+			if err != nil {
+				return fmt.Errorf("failed to parse msize for %q: %w", expanded, err)
+			}
+			options += fmt.Sprintf(",msize=%d", msize)
+			options += fmt.Sprintf(",cache=%s", *f.NineP.Cache)
+			// don't fail the boot, if virtfs is not available
+			options += ",nofail"
+		}
+		args.Mounts = append(args.Mounts, Mount{Tag: tag, Target: expanded, Type: fstype, Options: options})
+	}
+
+	switch *y.MountType {
+	case limayaml.REVSSHFS:
+		args.MountType = "reverse-sshfs"
+	case limayaml.NINEP:
+		args.MountType = "9p"
 	}
 
 	slirpMACAddress := limayaml.MACAddress(instDir)
@@ -153,6 +186,28 @@ func GenerateISO9660(instDir, name string, y *limayaml.LimaYAML, udpDNSLocalPort
 		}
 	}
 
+	args.CACerts.RemoveDefaults = y.CACertificates.RemoveDefaults
+
+	for _, path := range y.CACertificates.Files {
+		expanded, err := localpathutil.Expand(path)
+		if err != nil {
+			return err
+		}
+
+		content, err := os.ReadFile(expanded)
+		if err != nil {
+			return err
+		}
+
+		cert := getCert(string(content))
+		args.CACerts.Trusted = append(args.CACerts.Trusted, cert)
+	}
+
+	for _, content := range y.CACertificates.Certs {
+		cert := getCert(content)
+		args.CACerts.Trusted = append(args.CACerts.Trusted, cert)
+	}
+
 	if err := ValidateTemplateArgs(args); err != nil {
 		return err
 	}
@@ -204,43 +259,22 @@ func GuestAgentBinary(arch string) (io.ReadCloser, error) {
 	if arch == "" {
 		return nil, errors.New("arch must be set")
 	}
-	self, err := os.Executable()
+	dir, err := usrlocalsharelima.Dir()
 	if err != nil {
 		return nil, err
 	}
-	selfSt, err := os.Stat(self)
-	if err != nil {
-		return nil, err
-	}
-	if selfSt.Mode()&fs.ModeSymlink != 0 {
-		self, err = os.Readlink(self)
-		if err != nil {
-			return nil, err
-		}
-	}
+	gaPath := filepath.Join(dir, "lima-guestagent.Linux-"+arch)
+	return os.Open(gaPath)
+}
 
-	// self:  /usr/local/bin/limactl
-	selfDir := filepath.Dir(self)
-	selfDirDir := filepath.Dir(selfDir)
-	candidates := []string{
-		// candidate 0:
-		// - self:  /Applications/Lima.app/Contents/MacOS/limactl
-		// - agent: /Applications/Lima.app/Contents/MacOS/lima-guestagent.Linux-x86_64
-		filepath.Join(selfDir, "lima-guestagent.Linux-"+arch),
-		// candidate 1:
-		// - self:  /usr/local/bin/limactl
-		// - agent: /usr/local/share/lima/lima-guestagent.Linux-x86_64
-		filepath.Join(selfDirDir, "share/lima/lima-guestagent.Linux-"+arch),
-		// TODO: support custom path
-	}
-	for _, candidate := range candidates {
-		if f, err := os.Open(candidate); err == nil {
-			return f, nil
-		} else if !errors.Is(err, os.ErrNotExist) {
-			return nil, err
+func getCert(content string) Cert {
+	lines := []string{}
+	for _, line := range strings.Split(content, "\n") {
+		if line == "" {
+			continue
 		}
+		lines = append(lines, strings.TrimSpace(line))
 	}
-
-	return nil, fmt.Errorf("failed to find \"lima-guestagent.Linux-%s\" binary for %q, attempted %v",
-		arch, self, candidates)
+	// return lines
+	return Cert{Lines: lines}
 }
