@@ -19,6 +19,15 @@ var templateFS embed.FS
 
 const templateFSRoot = "cidata.TEMPLATE.d"
 
+type CACerts struct {
+	RemoveDefaults *bool
+	Trusted        []Cert
+}
+
+type Cert struct {
+	Lines []string
+}
+
 type Containerd struct {
 	System bool
 	User   bool
@@ -27,13 +36,20 @@ type Network struct {
 	MACAddress string
 	Interface  string
 }
+type Mount struct {
+	Tag     string
+	Target  string // abs path, accessible by the User
+	Type    string
+	Options string
+}
 type TemplateArgs struct {
 	Name            string // instance name
 	IID             string // instance id
 	User            string // user name
 	UID             int
 	SSHPubKeys      []string
-	Mounts          []string // abs path, accessible by the User
+	Mounts          []Mount
+	MountType       string
 	Containerd      Containerd
 	Networks        []Network
 	SlirpNICName    string
@@ -44,6 +60,7 @@ type TemplateArgs struct {
 	TCPDNSLocalPort int
 	Env             map[string]string
 	DNSAddresses    []string
+	CACerts         CACerts
 }
 
 func ValidateTemplateArgs(args TemplateArgs) error {
@@ -62,7 +79,8 @@ func ValidateTemplateArgs(args TemplateArgs) error {
 	if len(args.SSHPubKeys) == 0 {
 		return errors.New("field SSHPubKeys must be set")
 	}
-	for i, f := range args.Mounts {
+	for i, m := range args.Mounts {
+		f := m.Target
 		if !filepath.IsAbs(f) {
 			return fmt.Errorf("field mounts[%d] must be absolute, got %q", i, f)
 		}
