@@ -10,6 +10,7 @@ import (
 	"path/filepath"
 	"runtime"
 	"strconv"
+	"strings"
 	"text/template"
 
 	"github.com/lima-vm/lima/pkg/guestagent/api"
@@ -20,8 +21,16 @@ import (
 	"github.com/xorcare/pointer"
 )
 
+const (
+	Default9pSecurityModel   string = "mapped-xattr"
+	Default9pProtocolVersion string = "9p2000.L"
+	Default9pMsize           string = "128KiB"
+	Default9pCacheForRO      string = "fscache"
+	Default9pCacheForRW      string = "mmap"
+)
+
 func defaultContainerdArchives() []File {
-	const nerdctlVersion = "0.16.1"
+	const nerdctlVersion = "0.19.0"
 	location := func(goarch string) string {
 		return "https://github.com/containerd/nerdctl/releases/download/v" + nerdctlVersion + "/nerdctl-full-" + nerdctlVersion + "-linux-" + goarch + ".tar.gz"
 	}
@@ -29,12 +38,12 @@ func defaultContainerdArchives() []File {
 		{
 			Location: location("amd64"),
 			Arch:     X8664,
-			Digest:   "sha256:25b6c9a7059e568238f07baacc1ece6af3d698ae8d33cb9f0ec1e5161230ab62",
+			Digest:   "sha256:440c90a37e3bfc5d1dfa12bec5b65ed22bc479e13a5e141a69e24ae55a883ae8",
 		},
 		{
 			Location: location("arm64"),
 			Arch:     AARCH64,
-			Digest:   "sha256:e47fd1a03545ce539005a7f8a29644cb2ab515012c1c82b8be0bbafbb1670699",
+			Digest:   "sha256:d07c5213a6789eb1d3d07febaa982a043611e8205264066388c43317924cf446",
 		},
 	}
 }
@@ -64,6 +73,7 @@ func MACAddress(uniqueID string) string {
 // - Mounts are appended in d, y, o order, but "merged" when the Location matches a previous entry;
 //   the highest priority Writable setting wins.
 // - DNS are picked from the highest priority where DNS is not empty.
+// - CACertificates Files and Certs are uniquely appended
 func FillDefault(y, d, o *LimaYAML, filePath string) {
 	if y.Arch == nil {
 		y.Arch = d.Arch
@@ -81,6 +91,33 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 		}
 	}
 
+	cpuType := map[Arch]string{
+		AARCH64: "cortex-a72",
+		// Since https://github.com/lima-vm/lima/pull/494, we use qemu64 cpu for better emulation of x86_64.
+		X8664: "qemu64",
+	}
+	for arch := range cpuType {
+		if IsNativeArch(arch) {
+			cpuType[arch] = "host"
+		}
+	}
+	for k, v := range d.CPUType {
+		if len(v) > 0 {
+			cpuType[k] = v
+		}
+	}
+	for k, v := range y.CPUType {
+		if len(v) > 0 {
+			cpuType[k] = v
+		}
+	}
+	for k, v := range o.CPUType {
+		if len(v) > 0 {
+			cpuType[k] = v
+		}
+	}
+	y.CPUType = cpuType
+
 	if y.CPUs == nil {
 		y.CPUs = d.CPUs
 	}
@@ -161,6 +198,19 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 		y.SSH.ForwardAgent = pointer.Bool(false)
 	}
 
+	hosts := make(map[string]string)
+	// Values can be either names or IP addresses. Name values are canonicalized in the hostResolver.
+	for k, v := range d.HostResolver.Hosts {
+		hosts[Cname(k)] = v
+	}
+	for k, v := range y.HostResolver.Hosts {
+		hosts[Cname(k)] = v
+	}
+	for k, v := range o.HostResolver.Hosts {
+		hosts[Cname(k)] = v
+	}
+	y.HostResolver.Hosts = hosts
+
 	y.Provision = append(append(o.Provision, y.Provision...), d.Provision...)
 	for i := range y.Provision {
 		provision := &y.Provision[i]
@@ -326,6 +376,21 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 			if mount.SSHFS.FollowSymlinks != nil {
 				mounts[i].SSHFS.FollowSymlinks = mount.SSHFS.FollowSymlinks
 			}
+			if mount.SSHFS.SFTPDriver != nil {
+				mounts[i].SSHFS.SFTPDriver = mount.SSHFS.SFTPDriver
+			}
+			if mount.NineP.SecurityModel != nil {
+				mounts[i].NineP.SecurityModel = mount.NineP.SecurityModel
+			}
+			if mount.NineP.ProtocolVersion != nil {
+				mounts[i].NineP.ProtocolVersion = mount.NineP.ProtocolVersion
+			}
+			if mount.NineP.Msize != nil {
+				mounts[i].NineP.Msize = mount.NineP.Msize
+			}
+			if mount.NineP.Cache != nil {
+				mounts[i].NineP.Cache = mount.NineP.Cache
+			}
 			if mount.Writable != nil {
 				mounts[i].Writable = mount.Writable
 			}
@@ -344,9 +409,38 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 		if mount.SSHFS.FollowSymlinks == nil {
 			mount.SSHFS.FollowSymlinks = pointer.Bool(false)
 		}
+		if mount.SSHFS.SFTPDriver == nil {
+			mount.SSHFS.SFTPDriver = pointer.String("")
+		}
+		if mount.NineP.SecurityModel == nil {
+			mounts[i].NineP.SecurityModel = pointer.String(Default9pSecurityModel)
+		}
+		if mount.NineP.ProtocolVersion == nil {
+			mounts[i].NineP.ProtocolVersion = pointer.String(Default9pProtocolVersion)
+		}
+		if mount.NineP.Msize == nil {
+			mounts[i].NineP.Msize = pointer.String(Default9pMsize)
+		}
 		if mount.Writable == nil {
 			mount.Writable = pointer.Bool(false)
 		}
+		if mount.NineP.Cache == nil {
+			if *mount.Writable {
+				mounts[i].NineP.Cache = pointer.String(Default9pCacheForRW)
+			} else {
+				mounts[i].NineP.Cache = pointer.String(Default9pCacheForRO)
+			}
+		}
+	}
+
+	if y.MountType == nil {
+		y.MountType = d.MountType
+	}
+	if o.MountType != nil {
+		y.MountType = o.MountType
+	}
+	if y.MountType == nil || *y.MountType == "" {
+		y.MountType = pointer.String(REVSSHFS)
 	}
 
 	// Note: DNS lists are not combined; highest priority setting is picked
@@ -368,6 +462,22 @@ func FillDefault(y, d, o *LimaYAML, filePath string) {
 		env[k] = v
 	}
 	y.Env = env
+
+	if y.CACertificates.RemoveDefaults == nil {
+		y.CACertificates.RemoveDefaults = d.CACertificates.RemoveDefaults
+	}
+	if o.CACertificates.RemoveDefaults != nil {
+		y.CACertificates.RemoveDefaults = o.CACertificates.RemoveDefaults
+	}
+	if y.CACertificates.RemoveDefaults == nil {
+		y.CACertificates.RemoveDefaults = pointer.Bool(false)
+	}
+
+	caFiles := unique(append(append(d.CACertificates.Files, y.CACertificates.Files...), o.CACertificates.Files...))
+	y.CACertificates.Files = caFiles
+
+	caCerts := unique(append(append(d.CACertificates.Certs, y.CACertificates.Certs...), o.CACertificates.Certs...))
+	y.CACertificates.Certs = caCerts
 }
 
 func FillPortForwardDefaults(rule *PortForward, instDir string) {
@@ -375,7 +485,11 @@ func FillPortForwardDefaults(rule *PortForward, instDir string) {
 		rule.Proto = TCP
 	}
 	if rule.GuestIP == nil {
-		rule.GuestIP = api.IPv4loopback1
+		if rule.GuestIPMustBeZero {
+			rule.GuestIP = net.IPv4zero
+		} else {
+			rule.GuestIP = api.IPv4loopback1
+		}
 	}
 	if rule.HostIP == nil {
 		rule.HostIP = api.IPv4loopback1
@@ -461,3 +575,29 @@ func ResolveArch(s *string) Arch {
 	}
 	return *s
 }
+
+func IsNativeArch(arch Arch) bool {
+	nativeX8664 := arch == X8664 && runtime.GOARCH == "amd64"
+	nativeAARCH64 := arch == AARCH64 && runtime.GOARCH == "arm64"
+	return nativeX8664 || nativeAARCH64
+}
+
+func Cname(host string) string {
+	host = strings.ToLower(host)
+	if !strings.HasSuffix(host, ".") {
+		host += "."
+	}
+	return host
+}
+
+func unique(s []string) []string {
+	keys := make(map[string]bool)
+	list := []string{}
+	for _, entry := range s {
+		if _, found := keys[entry]; !found {
+			keys[entry] = true
+			list = append(list, entry)
+		}
+	}
+	return list
+}
