@@ -47,7 +47,11 @@ func TestFillDefault(t *testing.T) {
 
 	// Builtin default values
 	builtin := LimaYAML{
-		Arch:   pointer.String(arch),
+		Arch: pointer.String(arch),
+		CPUType: map[Arch]string{
+			AARCH64: "cortex-a72",
+			X8664:   "qemu64",
+		},
 		CPUs:   pointer.Int(4),
 		Memory: pointer.String("4GiB"),
 		Disk:   pointer.String("100GiB"),
@@ -72,7 +76,11 @@ func TestFillDefault(t *testing.T) {
 			IPv6:    pointer.Bool(false),
 		},
 		PropagateProxyEnv: pointer.Bool(true),
+		CACertificates: CACertificates{
+			RemoveDefaults: pointer.Bool(false),
+		},
 	}
+	builtin.CPUType[arch] = "host"
 
 	defaultPortForward := PortForward{
 		GuestIP:        api.IPv4loopback1,
@@ -88,9 +96,15 @@ func TestFillDefault(t *testing.T) {
 	// All these slices and maps are empty in "builtin". Add minimal entries here to see that
 	// their values are retained and defaults for their fields are applied correctly.
 	y = LimaYAML{
+		HostResolver: HostResolver{
+			Hosts: map[string]string{
+				"MY.Host": "host.lima.internal",
+			},
+		},
 		Mounts: []Mount{
 			{Location: "/tmp"},
 		},
+		MountType: pointer.String(NINEP),
 		Provision: []Provision{
 			{Script: "#!/bin/true"},
 		},
@@ -115,15 +129,32 @@ func TestFillDefault(t *testing.T) {
 		Env: map[string]string{
 			"ONE": "Eins",
 		},
+		CACertificates: CACertificates{
+			Files: []string{"ca.crt"},
+			Certs: []string{
+				"-----BEGIN CERTIFICATE-----\nYOUR-ORGS-TRUSTED-CA-CERT\n-----END CERTIFICATE-----\n",
+			},
+		},
 	}
 
 	expect := builtin
+	expect.HostResolver.Hosts = map[string]string{
+		"my.host.": "host.lima.internal",
+	}
+
 	expect.Mounts = y.Mounts
 	expect.Mounts[0].Writable = pointer.Bool(false)
 	expect.Mounts[0].SSHFS.Cache = pointer.Bool(true)
 	expect.Mounts[0].SSHFS.FollowSymlinks = pointer.Bool(false)
+	expect.Mounts[0].SSHFS.SFTPDriver = pointer.String("")
+	expect.Mounts[0].NineP.SecurityModel = pointer.String(Default9pSecurityModel)
+	expect.Mounts[0].NineP.ProtocolVersion = pointer.String(Default9pProtocolVersion)
+	expect.Mounts[0].NineP.Msize = pointer.String(Default9pMsize)
+	expect.Mounts[0].NineP.Cache = pointer.String(Default9pCacheForRO)
 	// Only missing Mounts field is Writable, and the default value is also the null value: false
 
+	expect.MountType = pointer.String(NINEP)
+
 	expect.Provision = y.Provision
 	expect.Provision[0].Mode = ProvisionModeSystem
 
@@ -158,6 +189,14 @@ func TestFillDefault(t *testing.T) {
 
 	expect.Env = y.Env
 
+	expect.CACertificates = CACertificates{
+		RemoveDefaults: pointer.Bool(false),
+		Files:          []string{"ca.crt"},
+		Certs: []string{
+			"-----BEGIN CERTIFICATE-----\nYOUR-ORGS-TRUSTED-CA-CERT\n-----END CERTIFICATE-----\n",
+		},
+	}
+
 	FillDefault(&y, &LimaYAML{}, &LimaYAML{}, filePath)
 	assert.DeepEqual(t, &y, &expect, opts...)
 
@@ -168,7 +207,11 @@ func TestFillDefault(t *testing.T) {
 
 	// Choose values that are different from the "builtin" defaults
 	d = LimaYAML{
-		Arch:   pointer.String("unknown"),
+		Arch: pointer.String("unknown"),
+		CPUType: map[Arch]string{
+			AARCH64: "arm64",
+			X8664:   "amd64",
+		},
 		CPUs:   pointer.Int(7),
 		Memory: pointer.String("5GiB"),
 		Disk:   pointer.String("105GiB"),
@@ -193,6 +236,9 @@ func TestFillDefault(t *testing.T) {
 		HostResolver: HostResolver{
 			Enabled: pointer.Bool(false),
 			IPv6:    pointer.Bool(true),
+			Hosts: map[string]string{
+				"default": "localhost",
+			},
 		},
 		PropagateProxyEnv: pointer.Bool(false),
 
@@ -239,6 +285,12 @@ func TestFillDefault(t *testing.T) {
 			"ONE": "one",
 			"TWO": "two",
 		},
+		CACertificates: CACertificates{
+			RemoveDefaults: pointer.Bool(true),
+			Certs: []string{
+				"-----BEGIN CERTIFICATE-----\nYOUR-ORGS-TRUSTED-CA-CERT\n-----END CERTIFICATE-----\n",
+			},
+		},
 	}
 
 	expect = d
@@ -246,6 +298,19 @@ func TestFillDefault(t *testing.T) {
 	expect.Containerd.Archives[0].Arch = *d.Arch
 	expect.Mounts[0].SSHFS.Cache = pointer.Bool(true)
 	expect.Mounts[0].SSHFS.FollowSymlinks = pointer.Bool(false)
+	expect.Mounts[0].SSHFS.SFTPDriver = pointer.String("")
+	expect.Mounts[0].NineP.SecurityModel = pointer.String(Default9pSecurityModel)
+	expect.Mounts[0].NineP.ProtocolVersion = pointer.String(Default9pProtocolVersion)
+	expect.Mounts[0].NineP.Msize = pointer.String(Default9pMsize)
+	expect.Mounts[0].NineP.Cache = pointer.String(Default9pCacheForRO)
+	expect.HostResolver.Hosts = map[string]string{
+		"default.": d.HostResolver.Hosts["default"],
+	}
+	expect.MountType = pointer.String(REVSSHFS)
+	expect.CACertificates.RemoveDefaults = pointer.Bool(true)
+	expect.CACertificates.Certs = []string{
+		"-----BEGIN CERTIFICATE-----\nYOUR-ORGS-TRUSTED-CA-CERT\n-----END CERTIFICATE-----\n",
+	}
 
 	y = LimaYAML{}
 	FillDefault(&y, &d, &LimaYAML{}, filePath)
@@ -268,6 +333,8 @@ func TestFillDefault(t *testing.T) {
 	expect.Mounts = append(d.Mounts, y.Mounts...)
 	expect.Networks = append(d.Networks, y.Networks...)
 
+	expect.HostResolver.Hosts["default."] = d.HostResolver.Hosts["default"]
+
 	// d.DNS will be ignored, and not appended to y.DNS
 
 	// "TWO" does not exist in filledDefaults.Env, so is set from d.Env
@@ -280,7 +347,11 @@ func TestFillDefault(t *testing.T) {
 	// User-provided overrides should override user-provided config settings
 
 	o = LimaYAML{
-		Arch:   pointer.String(arch),
+		Arch: pointer.String(arch),
+		CPUType: map[Arch]string{
+			AARCH64: "uber-arm",
+			X8664:   "pentium",
+		},
 		CPUs:   pointer.Int(12),
 		Memory: pointer.String("7GiB"),
 		Disk:   pointer.String("117GiB"),
@@ -309,6 +380,9 @@ func TestFillDefault(t *testing.T) {
 		HostResolver: HostResolver{
 			Enabled: pointer.Bool(false),
 			IPv6:    pointer.Bool(false),
+			Hosts: map[string]string{
+				"override.": "underflow",
+			},
 		},
 		PropagateProxyEnv: pointer.Bool(false),
 
@@ -320,6 +394,12 @@ func TestFillDefault(t *testing.T) {
 					Cache:          pointer.Bool(false),
 					FollowSymlinks: pointer.Bool(true),
 				},
+				NineP: NineP{
+					SecurityModel:   pointer.String("mapped-file"),
+					ProtocolVersion: pointer.String("9p2000"),
+					Msize:           pointer.String("8KiB"),
+					Cache:           pointer.String("none"),
+				},
 			},
 		},
 		Provision: []Provision{
@@ -362,6 +442,9 @@ func TestFillDefault(t *testing.T) {
 			"TWO":   "deux",
 			"THREE": "trois",
 		},
+		CACertificates: CACertificates{
+			RemoveDefaults: pointer.Bool(true),
+		},
 	}
 
 	y = filledDefaults
@@ -373,11 +456,20 @@ func TestFillDefault(t *testing.T) {
 	expect.PortForwards = append(append(o.PortForwards, y.PortForwards...), d.PortForwards...)
 	expect.Containerd.Archives = append(append(o.Containerd.Archives, y.Containerd.Archives...), d.Containerd.Archives...)
 
+	expect.HostResolver.Hosts["default."] = d.HostResolver.Hosts["default"]
+	expect.HostResolver.Hosts["my.host."] = d.HostResolver.Hosts["host.lima.internal"]
+
 	// o.Mounts just makes d.Mounts[0] writable because the Location matches
 	expect.Mounts = append(d.Mounts, y.Mounts...)
 	expect.Mounts[0].Writable = pointer.Bool(true)
 	expect.Mounts[0].SSHFS.Cache = pointer.Bool(false)
 	expect.Mounts[0].SSHFS.FollowSymlinks = pointer.Bool(true)
+	expect.Mounts[0].NineP.SecurityModel = pointer.String("mapped-file")
+	expect.Mounts[0].NineP.ProtocolVersion = pointer.String("9p2000")
+	expect.Mounts[0].NineP.Msize = pointer.String("8KiB")
+	expect.Mounts[0].NineP.Cache = pointer.String("none")
+
+	expect.MountType = pointer.String(NINEP)
 
 	// o.Networks[1] is overriding the d.Networks[0].Lima entry for the "def0" interface
 	expect.Networks = append(append(d.Networks, y.Networks...), o.Networks[0])
@@ -391,6 +483,12 @@ func TestFillDefault(t *testing.T) {
 	// ONE remains from filledDefaults.Env; the rest are set from o
 	expect.Env["ONE"] = y.Env["ONE"]
 
+	expect.CACertificates.RemoveDefaults = pointer.Bool(true)
+	expect.CACertificates.Files = []string{"ca.crt"}
+	expect.CACertificates.Certs = []string{
+		"-----BEGIN CERTIFICATE-----\nYOUR-ORGS-TRUSTED-CA-CERT\n-----END CERTIFICATE-----\n",
+	}
+
 	FillDefault(&y, &d, &o, filePath)
 	assert.DeepEqual(t, &y, &expect, opts...)
 }
