@@ -50,6 +50,15 @@ func Validate(y LimaYAML, warn bool) error {
 		}
 	}
 
+	for arch := range y.CPUType {
+		switch arch {
+		case AARCH64, X8664:
+			// these are the only supported architectures
+		default:
+			return fmt.Errorf("field `cpuType` uses unsupported arch %q", arch)
+		}
+	}
+
 	if *y.CPUs == 0 {
 		return errors.New("field `cpus` must be set")
 	}
@@ -93,6 +102,10 @@ func Validate(y LimaYAML, warn bool) error {
 		} else if !st.IsDir() {
 			return fmt.Errorf("field `mounts[%d].location` refers to a non-directory path: %q: %w", i, f.Location, err)
 		}
+
+		if _, err := units.RAMInBytes(*f.NineP.Msize); err != nil {
+			return fmt.Errorf("field `msize` has an invalid value: %w", err)
+		}
 	}
 
 	if *y.SSH.LocalPort != 0 {
@@ -101,6 +114,12 @@ func Validate(y LimaYAML, warn bool) error {
 		}
 	}
 
+	switch *y.MountType {
+	case REVSSHFS, NINEP:
+	default:
+		return fmt.Errorf("field `mountType` must be %q or %q , got %q", REVSSHFS, NINEP, *y.MountType)
+	}
+
 	// y.Firmware.LegacyBIOS is ignored for aarch64, but not a fatal error.
 
 	for i, p := range y.Provision {
@@ -125,6 +144,9 @@ func Validate(y LimaYAML, warn bool) error {
 	}
 	for i, rule := range y.PortForwards {
 		field := fmt.Sprintf("portForwards[%d]", i)
+		if rule.GuestIPMustBeZero && !rule.GuestIP.Equal(net.IPv4zero) {
+			return fmt.Errorf("field `%s.guestIPMustBeZero` can only be true when field `%s.guestIP` is 0.0.0.0", field, field)
+		}
 		if rule.GuestPort != 0 {
 			if rule.GuestSocket != "" {
 				return fmt.Errorf("field `%s.guestPort` must be 0 when field `%s.guestSocket` is set", field, field)
