@@ -12,10 +12,12 @@ import (
 	"strconv"
 	"strings"
 
+	"github.com/coreos/go-semver/semver"
 	"github.com/docker/go-units"
 	"github.com/lima-vm/lima/pkg/downloader"
 	"github.com/lima-vm/lima/pkg/iso9660util"
 	"github.com/lima-vm/lima/pkg/limayaml"
+	"github.com/lima-vm/lima/pkg/localpathutil"
 	"github.com/lima-vm/lima/pkg/networks"
 	qemu "github.com/lima-vm/lima/pkg/qemu/const"
 	"github.com/lima-vm/lima/pkg/qemu/imgutil"
@@ -142,7 +144,7 @@ func appendArgsIfNoConflict(args []string, k, v string) []string {
 }
 
 type features struct {
-	// NetdevHelp is the output of `qemu-system-x86_64 -accel help`
+	// AccelHelp is the output of `qemu-system-x86_64 -accel help`
 	// e.g. "Accelerators supported in QEMU binary:\ntcg\nhax\nhvf\n"
 	// Not machine-readable, but checking strings.Contains() should be fine.
 	AccelHelp []byte
@@ -150,6 +152,13 @@ type features struct {
 	// e.g. "Available netdev backend types:\nsocket\nhubport\ntap\nuser\nvde\nbridge\vhost-user\n"
 	// Not machine-readable, but checking strings.Contains() should be fine.
 	NetdevHelp []byte
+	// MachineHelp is the output of `qemu-system-x86_64 -machine help`
+	// e.g. "Supported machines are:\nakita...\n...virt-6.2...\n...virt-7.0...\n...\n"
+	// Not machine-readable, but checking strings.Contains() should be fine.
+	MachineHelp []byte
+
+	// VersionGEQ7 is true when the QEMU version seems v7.0.0 or later
+	VersionGEQ7 bool
 }
 
 func inspectFeatures(exe string) (*features, error) {
@@ -181,9 +190,120 @@ func inspectFeatures(exe string) (*features, error) {
 			f.NetdevHelp = stderr.Bytes()
 		}
 	}
+
+	cmd = exec.Command(exe, "-machine", "help")
+	cmd.Stdout = &stdout
+	cmd.Stderr = &stderr
+	if err := cmd.Run(); err != nil {
+		logrus.Warnf("failed to run %v: stdout=%q, stderr=%q", cmd.Args, stdout.String(), stderr.String())
+	} else {
+		f.MachineHelp = stdout.Bytes()
+		if len(f.MachineHelp) == 0 {
+			f.MachineHelp = stderr.Bytes()
+		}
+	}
+	f.VersionGEQ7 = strings.Contains(string(f.MachineHelp), "-7.0")
+
 	return &f, nil
 }
 
+// showDarwinARM64HVFQEMU620Warning shows a warning on M1 macOS when QEMU is older than 6.2.0_1.
+//
+// See:
+// - https://gitlab.com/qemu-project/qemu/-/issues/899
+// - https://github.com/Homebrew/homebrew-core/pull/96743
+// - https://github.com/lima-vm/lima/issues/712
+func showDarwinARM64HVFQEMU620Warning(exe, accel string, features *features) {
+	if runtime.GOOS != "darwin" {
+		return
+	}
+	if runtime.GOARCH != "arm64" {
+		return
+	}
+	if accel != "hvf" {
+		return
+	}
+	if features.VersionGEQ7 {
+		return
+	}
+	if exeFull, err := exec.LookPath(exe); err == nil {
+		if exeResolved, err2 := filepath.EvalSymlinks(exeFull); err2 == nil {
+			if strings.Contains(exeResolved, "Cellar/qemu/6.2.0_") {
+				// Homebrew's QEMU 6.2.0_1 or later
+				return
+			}
+		}
+	}
+	w := "This version of QEMU might not be able to boot recent Linux guests on M1 macOS hosts."
+	if _, err := exec.LookPath("brew"); err == nil {
+		w += "Run `brew upgrade` and make sure your QEMU version is 6.2.0_1 or later."
+	} else {
+		w += `Reinstall QEMU with the following commits (included in QEMU 7.0.0):
+- https://github.com/qemu/qemu/commit/ad99f64f "hvf: arm: Use macros for sysreg shift/masking"
+- https://github.com/qemu/qemu/commit/7f6c295c "hvf: arm: Handle unknown ID registers as RES0"
+`
+		w += "See https://github.com/Homebrew/homebrew-core/pull/96743 for the further information."
+	}
+	logrus.Warn(w)
+}
+
+func getMacOSProductVersion() (*semver.Version, error) {
+	cmd := exec.Command("sw_vers", "-productVersion")
+	// output is like "12.3.1\n"
+	b, err := cmd.Output()
+	if err != nil {
+		return nil, fmt.Errorf("failed to execute %v: %w", cmd.Args, err)
+	}
+	verTrimmed := strings.TrimSpace(string(b))
+	verSem, err := semver.NewVersion(verTrimmed)
+	if err != nil {
+		return nil, fmt.Errorf("failed to parse macOS version %q: %w", verTrimmed, err)
+	}
+	return verSem, nil
+}
+
+// adjustMemBytesDarwinARM64HVF adjusts the memory to be <= 3 GiB, only when the following conditions are met:
+//
+// - Host OS   <  macOS 12.4
+// - Host Arch == arm64
+// - Accel     == hvf
+// - QEMU      >= 7.0
+//
+// This adjustment is required for avoiding host kernel panic. The issue was fixed in macOS 12.4 Beta 1.
+// See https://github.com/lima-vm/lima/issues/795 https://gitlab.com/qemu-project/qemu/-/issues/903#note_911000975
+func adjustMemBytesDarwinARM64HVF(memBytes int64, accel string, features *features) int64 {
+	const safeSize = 3 * 1024 * 1024 * 1024 // 3 GiB
+	if memBytes <= safeSize {
+		return memBytes
+	}
+	if runtime.GOOS != "darwin" {
+		return memBytes
+	}
+	if runtime.GOARCH != "arm64" {
+		return memBytes
+	}
+	if accel != "hvf" {
+		return memBytes
+	}
+	if !features.VersionGEQ7 {
+		return memBytes
+	}
+	macOSProductVersion, err := getMacOSProductVersion()
+	if err != nil {
+		logrus.Warn(err)
+		return memBytes
+	}
+	if !macOSProductVersion.LessThan(*semver.New("12.4.0")) {
+		return memBytes
+	}
+	logrus.Warnf("Reducing the guest memory from %s to %s, to avoid host kernel panic on macOS <= 12.3 with QEMU >= 7.0; "+
+		"Please update macOS to 12.4 or later, or downgrade QEMU to 6.2; "+
+		"See https://github.com/lima-vm/lima/issues/795 for the further background.",
+		units.BytesSize(float64(memBytes)), units.BytesSize(float64(safeSize)))
+	memBytes = safeSize
+	return memBytes
+}
+
 func Cmdline(cfg Config) (string, []string, error) {
 	y := cfg.LimaYAML
 	exe, args, err := getExe(*y.Arch)
@@ -199,23 +319,24 @@ func Cmdline(cfg Config) (string, []string, error) {
 	// Architecture
 	accel := getAccel(*y.Arch)
 	if !strings.Contains(string(features.AccelHelp), accel) {
-		errStr := fmt.Sprintf("accelerator %q is not supported by %s", accel, exe)
-		if accel == "hvf" && *y.Arch == limayaml.AARCH64 {
-			errStr += " ( Hint: as of August 2021, qemu-system-aarch64 on ARM Mac needs to be patched for enabling hvf accelerator,"
-			errStr += " see https://gist.github.com/nrjdalal/e70249bb5d2e9d844cc203fd11f74c55 )"
-		}
-		return "", nil, errors.New(errStr)
+		return "", nil, fmt.Errorf("accelerator %q is not supported by %s", accel, exe)
 	}
+	showDarwinARM64HVFQEMU620Warning(exe, accel, features)
+
+	// Memory
+	memBytes, err := units.RAMInBytes(*y.Memory)
+	if err != nil {
+		return "", nil, err
+	}
+	memBytes = adjustMemBytesDarwinARM64HVF(memBytes, accel, features)
+	args = appendArgsIfNoConflict(args, "-m", strconv.Itoa(int(memBytes>>20)))
+
+	// CPU
+	cpu := y.CPUType[*y.Arch]
+	args = appendArgsIfNoConflict(args, "-cpu", cpu)
 	switch *y.Arch {
 	case limayaml.X8664:
-		cpu := "qemu64"
-		if isNativeArch(*y.Arch) {
-			cpu = "host"
-		}
-		args = appendArgsIfNoConflict(args, "-cpu", cpu)
-		if isNativeArch(*y.Arch) {
-			args = appendArgsIfNoConflict(args, "-machine", "q35,accel="+accel)
-		} else {
+		if strings.HasPrefix(cpu, "qemu64") {
 			// use q35 machine with vmware io port disabled.
 			args = appendArgsIfNoConflict(args, "-machine", "q35,vmport=off")
 			// use tcg accelerator with multi threading with 512MB translation block size
@@ -225,27 +346,26 @@ func Cmdline(cfg Config) (string, []string, error) {
 			args = appendArgsIfNoConflict(args, "-accel", "tcg,thread=multi,tb-size=512")
 			// This will disable CPU S3 state.
 			args = append(args, "-global", "ICH9-LPC.disable_s3=1")
+		} else {
+			args = appendArgsIfNoConflict(args, "-machine", "q35,accel="+accel)
 		}
 	case limayaml.AARCH64:
-		cpu := "cortex-a72"
-		if isNativeArch(*y.Arch) {
-			cpu = "host"
+		machine := "virt,accel=" + accel
+		// QEMU >= 7.0 requires highmem=off NOT to be set, otherwise fails with "Addressing limited to 32 bits, but memory exceeds it by 1073741824 bytes"
+		// QEMU <  7.0 requires highmem=off to be set, otherwise fails with "VCPU supports less PA bits (36) than requested by the memory map (40)"
+		// https://github.com/lima-vm/lima/issues/680
+		// https://github.com/lima-vm/lima/pull/24
+		// But when the memory size is <= 3 GiB, we can always set highmem=off.
+		if !features.VersionGEQ7 || memBytes <= 3*1024*1024*1024 {
+			machine += ",highmem=off"
 		}
-		args = appendArgsIfNoConflict(args, "-cpu", cpu)
-		args = appendArgsIfNoConflict(args, "-machine", "virt,accel="+accel+",highmem=off")
+		args = appendArgsIfNoConflict(args, "-machine", machine)
 	}
 
 	// SMP
 	args = appendArgsIfNoConflict(args, "-smp",
 		fmt.Sprintf("%d,sockets=1,cores=%d,threads=1", *y.CPUs, *y.CPUs))
 
-	// Memory
-	memBytes, err := units.RAMInBytes(*y.Memory)
-	if err != nil {
-		return "", nil, err
-	}
-	args = appendArgsIfNoConflict(args, "-m", strconv.Itoa(int(memBytes>>20)))
-
 	// Firmware
 	legacyBIOS := *y.Firmware.LegacyBIOS
 	if legacyBIOS && *y.Arch != limayaml.X8664 {
@@ -357,7 +477,28 @@ func Cmdline(cfg Config) (string, []string, error) {
 	args = append(args, "-chardev", fmt.Sprintf("socket,id=%s,path=%s,server=on,wait=off,logfile=%s", serialChardev, serialSock, serialLog))
 	args = append(args, "-serial", "chardev:"+serialChardev)
 
-	// We also want to enable vsock and virtfs here, but QEMU does not support vsock and virtfs for macOS hosts
+	// We also want to enable vsock here, but QEMU does not support vsock for macOS hosts
+
+	if *y.MountType == limayaml.NINEP {
+		for i, f := range y.Mounts {
+			tag := fmt.Sprintf("mount%d", i)
+			expanded, err := localpathutil.Expand(f.Location)
+			if err != nil {
+				return "", nil, err
+			}
+			if err := os.MkdirAll(expanded, 0755); err != nil {
+				return "", nil, err
+			}
+			options := "local"
+			options += fmt.Sprintf(",mount_tag=%s", tag)
+			options += fmt.Sprintf(",path=%s", expanded)
+			options += fmt.Sprintf(",security_model=%s", *f.NineP.SecurityModel)
+			if !*f.Writable {
+				options += ",readonly"
+			}
+			args = append(args, "-virtfs", options)
+		}
+	}
 
 	// QMP
 	qmpSock := filepath.Join(cfg.InstanceDir, filenames.QMPSock)
@@ -396,14 +537,8 @@ func getExe(arch limayaml.Arch) (string, []string, error) {
 	return exe, args, nil
 }
 
-func isNativeArch(arch limayaml.Arch) bool {
-	nativeX8664 := arch == limayaml.X8664 && runtime.GOARCH == "amd64"
-	nativeAARCH64 := arch == limayaml.AARCH64 && runtime.GOARCH == "arm64"
-	return nativeX8664 || nativeAARCH64
-}
-
 func getAccel(arch limayaml.Arch) string {
-	if isNativeArch(arch) {
+	if limayaml.IsNativeArch(arch) {
 		switch runtime.GOOS {
 		case "darwin":
 			return "hvf"
@@ -432,6 +567,8 @@ func getFirmware(qemuExe string, arch limayaml.Arch) (string, error) {
 		candidates = append(candidates, "/usr/share/OVMF/OVMF_CODE.fd")
 		// openSUSE package "qemu-ovmf-x86_64"
 		candidates = append(candidates, "/usr/share/qemu/ovmf-x86_64-code.bin")
+		// Archlinux package "edk2-ovmf"
+		candidates = append(candidates, "/usr/share/edk2-ovmf/x64/OVMF_CODE.fd")
 	case limayaml.AARCH64:
 		// Debian package "qemu-efi-aarch64"
 		candidates = append(candidates, "/usr/share/AAVMF/AAVMF_CODE.fd")
